package org.apache.commons.lang3.concurrent.locks;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import java.util.Objects;
import java.util.concurrent.locks.Lock;
import java.util.function.Supplier;
import org.apache.commons.lang3.concurrent.locks.LockingVisitors;
import org.apache.commons.lang3.function.Failable;
import org.apache.commons.lang3.function.FailableConsumer;
import org.apache.commons.lang3.function.FailableFunction;

public class LockVisitor<O, L> {
  private final L lock;
  
  private final O object;
  
  private final Supplier<Lock> readLockSupplier;
  
  private final Supplier<Lock> writeLockSupplier;
  
  protected LockVisitor(O object, L lock, Supplier<Lock> readLockSupplier, Supplier<Lock> writeLockSupplier) {
    this.object = Objects.requireNonNull(object, "object");
    this.lock = Objects.requireNonNull(lock, "lock");
    this.readLockSupplier = Objects.<Supplier<Lock>>requireNonNull(readLockSupplier, "readLockSupplier");
    this.writeLockSupplier = Objects.<Supplier<Lock>>requireNonNull(writeLockSupplier, "writeLockSupplier");
  }
  
  public void acceptReadLocked(FailableConsumer<O, ?> consumer) {
    lockAcceptUnlock(this.readLockSupplier, consumer);
  }
  
  public void acceptWriteLocked(FailableConsumer<O, ?> consumer) {
    lockAcceptUnlock(this.writeLockSupplier, consumer);
  }
  
  public <T> T applyReadLocked(FailableFunction<O, T, ?> function) {
    return lockApplyUnlock(this.readLockSupplier, function);
  }
  
  public <T> T applyWriteLocked(FailableFunction<O, T, ?> function) {
    return lockApplyUnlock(this.writeLockSupplier, function);
  }
  
  public L getLock() {
    return this.lock;
  }
  
  public O getObject() {
    return this.object;
  }
  
  protected void lockAcceptUnlock(Supplier<Lock> lockSupplier, FailableConsumer<O, ?> consumer) {
    Lock lock = lockSupplier.get();
    lock.lock();
    try {
      consumer.accept(this.object);
    } catch (Throwable t) {
      throw Failable.rethrow(t);
    } finally {
      lock.unlock();
    } 
  }
  
  protected <T> T lockApplyUnlock(Supplier<Lock> lockSupplier, FailableFunction<O, T, ?> function) {
    Lock lock = lockSupplier.get();
    lock.lock();
    try {
      return function.apply(this.object);
    } catch (Throwable t) {
      throw Failable.rethrow(t);
    } finally {
      lock.unlock();
    } 
  }
}
